# Ledger.pm
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Ledger.pm,v 1.109 2006/05/17 01:35:54 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

=head1 NAME

Ledger - Object used to output Ledger screens

=head1 SYNOPSIS

  use Portal::Accounting::Ledger;
  my $obj = Portal::Accounting::Ledger->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Ledger is an accounting ledger class.

=cut
package Portal::Accounting::Ledger;
my $state = "Ledger";
use strict;
use Portal::Accounting::DB::Main;
use Portal::Accounting::BaseL;
use Portal::Accounting::Objects::JournalEntry;
use Portal::Accounting::Objects::Ledger;
use Portal::Accounting::Objects::CloseOut;
use Portal::Accounting::Objects::OpenBooks;
use HTMLObject::Form;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.109 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Ledger Summary",            #always checks for output variable
  closeLedger => "Close Ledger",
  'open' => "Open",
  ledgerFrame => "Ledger Frame",
  ledgerMenu  => "Leger Menu",
  displayCloseOut => "Close Books",
  closeOut => "Close Books",
  displayReOpen => "Open Books",
  reOpen => "Open Books",
);

=head1 Exported FUNCTIONS


B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(langObj)

 Creates a new instance of the Portal::Accounting::Ledger module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self;
  eval { $self = $class->SUPER::new(@_); };
  if ($@)
  {
    die $@;
    return $self;
  }
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # do validation
  if (!$self->Portal::Accounting::Ledger::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  $self->{commands} = \%commands;
  return $self;
}

=head2 bool isValid(void)

    Returns 0 or 1 to indicate if the object is valid.  The error will be
    available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # do validation code here.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub c_display
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $demo;
  my $mainDB = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->langObj);
  if ($mainDB->error)
  {
    $self->error("Could not initialize DB::Main object!<br>".$mainDB->errorMessage."\n");
    return undef;
  }
  my $je = Portal::Accounting::Objects::JournalEntry->new(dbHandle => $self->{dbHandle}, langObj => $self->langObj, ledgerCode => $ledgerCode);
  my $l = $je->{ledger};  #!!! probably not a good idea to cheat like this incase JE changes api later.
  my $doc = $self->prepDoc;

  my $fy = $l->workingYear;
  my $from = $l->start(fiscalYear => $fy, month => 1);
  my $to = $l->end(fiscalYear => $fy, month => 12);
  my $last = $l->lastClose;
  my $firstYear;
  if ($last eq '')
  {
    $last = "#LANG=Never#";
  }
  else
  {
    $firstYear = 'style="display: none; "'
  }

  $doc->setFocus("body");
  if ($self->demoCheck)
  {
    $demo .= '<div id="debug" style="display: none;"><div>Ledger month tests:<br>'.
      $l->start(1).":".
      $l->start(2).":".
      $l->start(3).":".
      $l->start(4).":".
      $l->start(5).":".
      $l->start(6).":".
      $l->start(7).":".
      $l->start(8).":".
      $l->start(9).":".
      $l->start(10).":".
      $l->start(11).":".
      $l->start(12)."<br>";
    $demo .=
      $l->end(1).":".
      $l->end(2).":".
      $l->end(3).":".
      $l->end(4).":".
      $l->end(5).":".
      $l->end(6).":".
      $l->end(7).":".
      $l->end(8).":".
      $l->end(9).":".
      $l->end(10).":".
      $l->end(11).":".
      $l->end(12)."</div>";
    my @start = $l->getQuarterStarts($fy);
    my @end = $l->getQuarterEnds($fy);
    $demo .= "<div>QUARTER STARTS<br>"
      . $start[0]. ":"
      . $start[1]. ":"
      . $start[2]. ":"
      . $start[3]. "</div>";
    $demo .= "<div>QUARTER ENDS<br>"
      . $end[0]. ":"
      . $end[1]. ":"
      . $end[2]. ":"
      . $end[3]. "</div>";
    $demo .= "Ledger year tests<br>";
    my %known = $l->selectKnownYears();
    if ($l->error) { $demo .= $l->errorMessage; }
    else { $demo .= "<div>".join(":", sort keys %known)."</div>"; }
    my $pre = $l->preOpenBooksDate;
    if ($l->error) { $demo .= $l->errorMessage; }
    else { $demo .= "<div>pre books date: $pre</div>"; }
    $demo .= '</div><span onclick="javascript: document.getElementById(\'debug\').style.display = \'block\';">&gt;&gt;</span>';
  }
  my $html .= <<"ENDOF";
$demo
<div style="float: right; " class="primary">
  <p>#LANG=Fiscal Year# $fy:<br>
  $from - $to</p>
  <p>#LANG=Last Close Out#:<br>
  $last
  </p>
  <ul>
    <li><a href="?$self->{getInfo}&command=displayCloseOut">#LANG=Close Books#</a></li>
    <li><a href="?$self->{getInfo}&command=displayReOpen">#LANG=Open Books#</a></li>
    <li $firstYear><a href="?app=$self->{app}&state=JournalEntry&command=opening&ledgerCode=$ledgerCode">#LANG=Opening Balances#</a></li>
  </ul>
</div>
<div>
  <style type="text/css">
  <!--
    .CashSummaryTitle { font-weight: bold; text-decoration: none; }
    .CashSummaryTable {  }
    .CashSummaryName { text-transform: capitalize; }
    .CashSummaryBalance {text-align: right; }
    .CashSummaryReconcile { }
    .CashSummaryLastEntry { }
    .negative { color: #ff0000; }
  -->
  </style>
ENDOF
  $html .= $mainDB->getCashAccountSummary(ledgerCode => $self->{input}->{ledgerCode});
  if ($mainDB->error)
  {
    $html .= $mainDB->errorMessage;
  }
  if (not defined $html)
  {
    $html .= "Error during Cash account Summary!<br>" . $mainDB->errorMessage();
  }
  $html .= <<"ENDOF";
</div>
  <hr noshade size="2">
  <iframe name="watchList" align="top" frameborder="0" width="100%" scrolling="yes" Height="600" src="?app=$self->{app}&state=Investments::WatchList&command=view&ledgerCode=$ledgerCode&groupName=SHORT"></iframe>
ENDOF
#!!! use ajax to fetch page instead of using iframe here.  will make page cleaner.
  $doc->print($self->langSubstitute($html));
  return $doc;
}

sub c_closeLedger
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my $errStr = "Portal::Accounting::Ledger->closeLedger()  - Error!<br>\n";
  my $siteDir = $self->{portalConfigObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $windowName = $self->{input}->{windowName};

  # remove this ledger from the window list.
  if ($self->{sessionObj}->windowExists($windowName))
  {
    $self->{sessionObj}->unregisterWindow($windowName);
  }
  if ($self->{sessionObj}->error)
  {
    $self->error($self->{sessionObj}->errorMessage);
    return undef;
  }

  # generate the document to close this window.
  my $onLoadCode = "window.close();\n";
  $doc->setOnload(code => $onLoadCode);
  $doc->setTitle($phrases->map("Close Window")." - $windowName");
  $doc->setFocus("body");
  $doc->print($phrases->map("Close Window")." - $windowName\n");

  return $doc;
}

sub c_ledgerMenu
{
  my $self = shift;
  my %args = (@_ );
  my $doc = $self->prepDoc(heading => '');
  my $errStr = "Portal::Accounting::Main->ledgerMenu()  - Error!<br>\n";
  my $phrases = $self->{langObj};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $ledger = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name};
  my $target = "";
  my $windowName;
  my $mainHeight = "600";
  my $mainWidth = "1024";

  my $url = $self->{baseURL};
  my $jsUrl = $self->{jsDir};

  if (exists $self->{input}->{windowName})
  {  $windowName = $self->{input}->{windowName}; }
 if ($windowName eq "")
 {
   $self->setError( errorString => "Did not receive a windowName for parent of frameset<br>\n");
   return undef;
 }
  $target = $windowName . "Data";
  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");
function getTimeStamp()
{
  var timeStamp = new Date();
  return timeStamp.getTime();
}

function openLedger(windowName)
{
  var foo = new Array();
  if (windowName == null || windowName == '') { windowName = getTimeStamp(); }
  foo.app = 'Accounting';
  foo.state = 'Ledger';
  foo.command = 'open';
  foo.ledgerCode = '$self->{input}->{ledgerCode}';
  foo.windowName = windowName;
  // toolbar is on because without it the tabs dont show up
  var options = ' width=$mainWidth,height=$mainHeight,scrollbars,status,location=no,menubar=no,copyhistory=no,directories=no,toolbar';
  myGet(windowName, '', foo, options);
}
END_OF_CODE

  my $newFrame = $phrases->map("New Frame");
  my $newW = $phrases->map("New Window");
  my $closeW = $phrases->map("Close Window");
  $doc->setBase(target => $target);
  $doc->setFocus("body");
  #  New frameset must use get bc of target=_top.  does not work as post
  #  X close window does not work in tabs.  only the first instance works for some reason.
  $doc->print(<<"END_OF_CODE");
<table width="100%">
  <tr>
    <td align="right">
      <font size="+2">
        <a href="?$self->{getInfo}&command=ledgerFrame" target="_top" title="$newFrame">F</a>
        | <a href="" onclick="javascript: openLedger(); return false;" title="$newW">W</a>
        | <a href="" onclick="myGet('$windowName', '', '$self->{getInfo}&command=closeLedger&ledgerCode=$ledgerCode'); return false;" title="$closeW" target="_top">X</a>
      </font>
    </td>
</table>
<br>
END_OF_CODE
  $doc->print("\n<ul>");
  $doc->print("\n <li> <a href=\"$url?command=display&$self->{getInfo}\">".$phrases->map("Summary")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=Accounts&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Chart of Accounts")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=Register&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Registers")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=JournalEntry&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Enter Transactions")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=JournalEntry&command=searchForm&ledgerCode=$ledgerCode\">".$phrases->map("Search")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=SubAccounts&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Sub Accounts")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=SubEntities&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Sub Entities")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=".$doc->encodeString('Payroll::Main')."&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Payroll")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=".$doc->encodeString('Investments::Main')."&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Investments")."</a></li>");
  $doc->print("\n <li> <a href=\"$url?app=$self->{app}&state=".$doc->encodeString('Reports::Main')."&command=display&ledgerCode=$ledgerCode\">".$phrases->map("Reports")."</a></li>");
  $doc->print("\n</ul>");

  return $doc;
}

=item c_open

  This routine is to be called when a new window has been opened for a ledger.
  It will register the window with the Portal then open the requested ledgerFrameSet.

  requires:  windowName, ledgerCode
=cut
sub c_open
{
  my $self = shift;
  $self->{sessionObj}->registerWindow($self->{input}->{windowName});
  if ($self->{sessionObj}->error)
  {
    $self->error($self->{sessionObj}->errorMessage);
    return undef;
  }
  return $self->c_ledgerFrame(@_);
}

sub c_ledgerFrame
{
  my $self = shift;
  my %args = ( @_ );
  my $errStr = "Portal::Accounting::Ledger->ledgerFrame()  - Error!<br>\n";
  my $phrases = $self->{langObj};
  my $framedoc = HTMLObject::FrameSet->new();
  my $windowName;
  my $menuSize = "150"; #should come from user prefs#!!!

  my $url = $self->{baseURL};

  if (exists $self->{input}->{windowName})
  {
    $windowName = $self->{input}->{windowName};
    if ($windowName eq "")
    {
      $self->setError(errorString => $errStr . "Window name was not set!<br>\n");
      return undef;
    }
  }
  else
  {
    $self->setError( $self->errorMessage() . "Did not receive a windowName for frameset<br>\n");
    return undef;
  }
  $framedoc->setTitle($self->{sessionObj}->{store}->{ledgers}->{$self->{input}->{ledgerCode}}->{name});

  my $inner_content;
  $inner_content = $framedoc->createNoframe($phrases->map("frameBrowser"));
  $inner_content .= $framedoc->createFrame(
    src => "$url?$self->{getInfo}&command=display",
    frameborder => 'yes',
    scrolling => 'auto',
    name => $windowName . "Data",
    bordercolor => 'white',
    marginwidth => '1');
  $inner_content .= $framedoc->createFrame(
    src => "$url?$self->{getInfo}&command=ledgerMenu",
    frameborder => 'yes', scrolling => 'auto',
    name => $windowName . "Menu",
    bordercolor => 'white',
    marginwidth => '0',
    marginheight => '0',
    border => '1');
  $framedoc->outerFrameset(
    cols => "*, $menuSize",
    name => $windowName,
    content => $inner_content,
    border => '3',
    frameborder => 'yes');
  return $framedoc;
}

sub c_displayReOpen
{
  my $self = shift;
  return $self->c_reOpen;
}

sub c_reOpen
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $reopen = Portal::Accounting::Objects::OpenBooks->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledgerCode => $self->{input}->{ledgerCode});

  if ($self->{input}->{command} eq "reOpen")
  {
    if ($reopen->populate(%{$self->{input}}))
    {
      my $result = $reopen->update;
      if ($reopen->error)
      {
        $doc->print($reopen->errorMessage);
      }
      else
      {
        $doc->print($result);
        $doc->print($self->mainButton);
        return $doc;
      }
    }
  }
  my %form = $reopen->printForm;
  if ($reopen->error) { $doc->print($reopen->errorMessage); }
  else { $doc->print(%form); }
  $doc->print($self->mainButton);
  return $doc;
}

sub c_displayCloseOut
{
  my $self = shift;
  return $self->c_closeOut;
}

sub c_closeOut
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $l = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle);
  $l->set('code', $ledgerCode);
  $l->select;
  if ($l->error) { $self->error($l->errorMessage); return $doc; }
  my $closeout = Portal::Accounting::Objects::CloseOut->new(langObj => $self->{langObj}, dbHandle => $self->dbHandle, ledger => $l);
  my $ledgerName = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name};
  my $phrases = $self->{langObj};
  my $result;

  if ($self->{input}->{command} eq "closeOut")
  {
    if ($closeout->validateForm(input => $self->{input}) && $closeout->populate(%{$self->{input}}))
    {
      my $result = $closeout->update;
      if ($closeout->error)
      {
        $doc->print($closeout->errorMessage);
      }
      else
      {
        $doc->print($closeout->debugString);
        $doc->print($result);
        $doc->print($self->mainButton);
        return $doc;
      }
    }
  }

  $closeout->resetError;
  $doc->print($closeout->printForm);
  if ($closeout->error )
  {
    $doc->print($closeout->errorMessage);
  }
  $doc->print($self->errorMessage);
  $doc->print($self->mainButton);
  return $doc;
}

1;
__END__

=head2

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

JT Moree
PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
