# ChartOfAccounts.pm - The Object Class that provides a ChartOfAccounts Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: ChartOfAccounts.pm,v 1.15 2006/04/19 20:34:56 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::ChartOfAccounts;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.15 $';

=head1 NAME

ChartOfAccounts - Object used to build a Chart Of Accounts.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::ChartOfAccounts;
  my $ChartOfAccountsObj = Portal::Accounting::Objects::ChartOfAccounts->new();
  if ($ChartOfAccountsObj->error())
  {
    die $ChartOfAccountsObj->errorMessage();
  }

=head1 DESCRIPTION

ChartOfAccounts is an object which holds, manages, displays, updates, edits account information and assignments.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(langObj =>, dbHandle => )

    Creates a new instance of Portal::Accounting::Objects::ChartOfAccounts.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments for parent.

    Optional:
      meta => 1,2,3,4,5 - will cause this object to only work with the specified metatypes.  Don't use spaces just comma seperators
      type => 1,33,42,5 - will cause this object to only work with the specified types.  Don't use spaces just comma seperators
        1 - assets, 2 - liabilities, 3- equity, 4 - revenues, 5 - expenses
      ledgerCode => "string" - will use this ledgerCode in urls
      app => "Accounting" - you might be able to re-use this object in another application
      state => "string" - you might be able to reuse this object in another state.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => "", type => "", meta => "", app => "Accounting", state => "Accounts", @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  my $l = $self->{langObj};
  $self->{metaTypes} = { 1 => $l->map("Asset"),
    2 => $l->map("Liability"),
    3 => $l->map("Equity"),
    4 => $l->map("Revenue"),
    5 => $l->map("Expense"),
  };

  my $active;
  if ($args{ledgerCode} ne '') { $active = 'display: none;'; }
  #why only 70% on the width?
  $self->{css}->{main} = <<"ENDOF";
  .chartOfAccounts { width: 70%; }
  .name     { text-align: left; }
  .code     { text-align: left; }
  .type     { text-align: left; }
  .active   { text-align: center; }
  .taxable  { text-align: center; }
  .assigned { text-align: center; }
  .active_yes   { }
  .active_no { $active }
ENDOF
  $self->{data}->{main}->{ledgerCode}->{-Value} = $args{ledgerCode};
  $self->{dbHandle} = $args{dbHandle};
  $self->{data}->{main}->{app}->{-Value} = $args{app};
  $self->{data}->{main}->{state}->{-Value} = $args{state};
  # instantiate anything unique to this module
  my $ledgerSql = " false ";
  if ($args{ledgerCode} ne '')
  {
    $ledgerSql = " (SELECT true FROM local_accounts_tb WHERE local_accounts_tb.acct_num = number AND local_accounts_tb.ledger_code = '$args{ledgerCode}' LIMIT 1) ";
  }
  my $sql = "SELECT name, code, number, type, taxable";
  $sql .= ", (SELECT first_digit FROM acct_type_tb WHERE num = type) AS first_digit, $ledgerSql AS assigned ";
  $sql .= " FROM account_tb ORDER BY first_digit, name, type";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return $self; }
  while (my $data = $result->fetchrow_hashref)
  {
    if ($args{ledgerCode} eq '' || $data->{assigned} eq '') { $data->{assigned} = '&nbsp;&nbsp;'; }
    elsif ($data->{assigned} eq '1') { $data->{assigned} = 'X'; }
    push (@{$self->{detail}}, $data);
  }
  $sql = "SELECT name, num FROM acct_type_tb";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return $self; }
  while (my $data = $result->fetchrow_hashref)
  {
    $self->{types}->{$data->{num}} = $self->langObj->map($data->{name});
  }
  # do anything else you might need to do.
  my $meta = $args{meta};
  if ($meta ne "")
  {
    if ($meta =~ /^([1-5]+,?)+$/)
    {
      $meta = "AND account_tb.type = acct_type_tb.num AND acct_type_tb.first_digit IN ($meta)";
    }
    else
    {
      $self->error("<p>Meta value is invalid: '$meta'</p>\n");
      $meta = "";
    }
  }
  my $type = $args{type};
  if ($type ne "")
  {
    if ($type =~ /^(\d+,?)+$/)
    {
      $type = "AND account_tb.type IN ($type)";
    }
    else
    {
      $self->error("<p>type value is invalid: '$type'</p>\n");
      $type = "";
    }
  }
  my $led = "";
  if ($self->{data}->{main}->{ledgerCode}->{-Value} ne "")
  {
    $led = "AND local_accounts_tb.ledger_code = '#FIELD=ledgerCode#'";
  }
  $self->{LIST}[0] = "SELECT account_tb.name||'-'||account_tb.code AS name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.acct_num = account_tb.number $led $meta $type ORDER BY name";
  $self->isValid;
  return $self;
}

=item sth selectSTH

  This routine uses the internal LIST variable to select from the database
  a list of accounts.  If the ledgerCode was specified to this object
  during new then only those accounts which are assigned to the specified
  ledger will be in the list.

=cut

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (! defined $self->{dbHandle})
  {
    $self->missing("dbHandle");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

#take the approach that this chart of accounts is mainly for assignement and to link to editing of accounts or new.
#if i do this on the client side i can simply
#
#1 send down raw data
#2 have client display data with markup etc.
#3 client can enable, disable assignment button as user changes ledger
sub printHTML
{
  my $self = shift;
  my $l = $self->langObj;
  my $click4new = $l->map("Create"). " ". $l->map("Account");
  my $click2edit = $l->map("Edit"). " ". $l->map("Account");
  my $meta = "";
  my $Name = $l->map("Account");
  my $Code = $l->map("Code");
  my $Type = $l->map("Type");
  my $Active = $l->map("Active");
  my $Taxable = $l->map("Taxable");
  my $Assigned = $l->map("Assign");
  my $showInActive = $l->map("Show InActive");
  my $hideInActive = $l->map("Hide InActive");
  my $click = $l->map("Click here to Assign");
  my $lc = $self->get('ledgerCode');

  my %return;
  $return{css} = $self->{css}->{main};
  $return{javascript} = <<"ENDOF";
ENDOF
  if ($lc eq '')
  {
    $return{css} .= ".assigned,.active { display: none; }";
  }
  my $app = $self->get('app');
  my $state = $self->get('state');
  $return{body} .= <<"ENDOF";
<form name="mainForm">
<input type="hidden" name="app" value="$app">
<input type="hidden" name="state" value="$state">
<input type="hidden" name="command" value="assign">
<input type="hidden" name="ledgerCode" value="$lc">
<input type="hidden" name="assign" value="">
<input type="hidden" name="unAssign" value="">
<table class="primary" style="width:95%">
  <thead class="tertiary">
    <tr>
      <th class="assigned"><input type="submit" name="submit" value="$Assigned" title="$click" /></th>
      <th class="tertiary active" style="text-align: center">
        $Active<input type="button" id="showIA1" title="$showInActive" onclick="showInActive();" value="+" /><input type="button" class="active_no" id="hideIA1" title="$hideInActive" onclick="hideInActive();" value="-" />
      </th>
      <th class="tertiary name">$Name</th>
      <th class="code">$Code</th>
      <th class="type">$Type</th>
      <th class="taxable">$Taxable</th>
    </tr>
  </thead>
  <tfoot> 
    <tr>
      <th class="assigned" ><input type="submit" name="submit" value="$Assigned" title="$click"></th>
      <th class="tertiary active" style="text-align: center">
        $Active<input type="button" id="showIA2" title="$showInActive" onclick="showInActive();" value="+" /><input type="button" class="active_no" id="hideIA2" title="$hideInActive" onclick="hideInActive();" value="-" />
      </th>
      <th class="name">$Name</th>
      <th class="code">$Code</th>
      <th class="type">$Type</th>
      <th class="taxable">$Taxable</th>
    </tr>
  </tfoot>
  <tbody>  
ENDOF
  foreach my $d (@{$self->{detail}})
  {
    if ($meta ne $d->{first_digit})
    {
      my $help = $self->langObj->map('metaHelp_'.$d->{first_digit});
      $return{body} .= <<"ENDOF";
 <tr class="secondary">
  <td class="name" colspan="6" style="font-size: big; font-weight: bold; padding-top: 1em;">
    <a title="$click4new" href="?app=$app&state=$state&command=new&ledgerCode=$lc&meta=$d->{first_digit}">$self->{metaTypes}->{$d->{first_digit}}</a>
    <span title="$help">?</span>
  </td>
 </tr>
ENDOF
    }
    $meta = $d->{first_digit}; #set to next group so that heading will only be output on changes
    my $class = "active_no";
    if ($d->{assigned} eq 'X') { $class = "active_yes";  }
    
    $return{body} .= " <tr class=\"$class\">\n";
    if ($self->get('ledgerCode') ne "")
    {
      if ($d->{assigned} eq 'X')
      {  #check the assigned button on
        $return{body} .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" onclick=\"toggleAssign($d->{number});\" checked=\"1\" /></td>\n";
        $return{body} .= "  <td class=\"active\">[$d->{assigned}]</td>\n";
      }
      else
      {  #check the assigned button off
        $return{body} .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" onclick=\"toggleAssign($d->{number});\" /></td>\n";
        $return{body} .= "  <td class=\"active\">[$d->{assigned}] </td>\n";
      }
    }
    else
    {
      #disalbe assign button
      $return{body} .= "  <td class=\"assigned\" ><input type=\"checkbox\" name=\"assign_$d->{number}\" onclick=\"toggleAssign($d->{number});\" disabled=\"1\" /></td>\n";
      $return{body} .= "  <td class=\"active\" >[ &nbsp;]</td>\n";
    }
    $return{body} .= "  <td class=\"name\"><a href=\"?app=$app&state=$state&command=edit&ledgerCode=$lc&meta=$d->{first_digit}&id=$d->{number}\" title=\"$click2edit\">$d->{name}</a></td>\n";
    $return{body} .= "  <td class=\"code\">$d->{code}</td>\n";
    $return{body} .= "  <td class=\"type\">$self->{types}->{$d->{type}}</td>\n";
    if ($d->{taxable} == 0) { $return{body} .= "  <td class=\"taxable\">[ &nbsp;]</td>\n" }
    else { $return{body} .= "  <td class=\"taxable\">[X]</td>\n" }
    $return{body} .= " </tr>\n";
  }
  my $Asset = $l->map("Asset");
  my $Liability = $l->map("Liability");
  my $Equity = $l->map("Equity");
  my $Revenue = $l->map("Revenue");
  my $Expense = $l->map("Expense");
  $return{body} .= <<"ENDO";
  </tbody>
</table>
</form>
  <div class="secondary"><form name="new">
  <input type="hidden" name="app" value="$app">
  <input type="hidden" name="state" value="$state">
  <input type="hidden" name="command" value="new">
  <input type="hidden" name="ledgerCode" value="$lc">
  <input type="radio" name="meta" value="1" checked>$Asset&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="2">$Liability&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="3">$Equity&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="4">$Revenue&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="5">$Expense&nbsp;&nbsp;&nbsp;
  <input type="submit" name="submit" value="$click4new">
  </div>
ENDO
  return %return;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Base(3)

=cut
