#$Id: JournalEntry.pm,v 1.61 2006/05/31 21:31:14 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::JournalEntry;

use strict;
use Portal::Accounting::Objects::JournalEntry::Row;
use Portal::Accounting::Objects::BaseL;
use Portal::Accounting::Objects::Ledger;
use Date::Calc qw(Date_to_Days);
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.61 $';
=head1 NAME

JournalEntry - Object used to retrieve, insert, and manipulate a journal transaction.

=head1 SYNOPSIS

  use JournalEntry;
  my $transobj = JournalEntry->new(langObj => . . .);
  $transobj->set("id","12345");
  $transObj->select();
  if ($transobj->error())
  {
    die $transobj->errorMessage();
  }

  valid data fields can be set with $transobj->set('field', 'value');
    "id" => { -Type => "hidden", -Value => '-1' },
    "journal" => {-Type => "hidden", -Value => "G" },
    "date" => {-Type => "hidden", },
    "payee" => {-Type => "hidden", },
    "remember" => {-Type => "hidden", -Value => "false"},
    "trackingId" => {-Type => "hidden", },
    "origId" => {-Type => "hidden", },
    "ledgerCode" => {-Type => "hidden", },
    "workingYear" => {-Type => "hidden", },
    "lastClose" => {-Type => "hidden", },

=head1 DESCRIPTION

JournalEntry is a class that stores data for an accounting
JournalEntry.  It provides a container for a JournalEntry that will be used
to write to the database or update an existing record in the database.

=head1 Exported FUNCTIONS

=item scalar new(langObj =>, ledger => )

    Creates a new instance of the Portal::Accounting::Objects::JournalEntry
    object.  A new instance is created in an invalid state (i.e., missing
    fields, etc.).  isValid() will be called on the object before using the
    object in a database JournalEntry.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => '', debug => 0, @_ );
  my $lc = $args{ledgerCode};
  my $lo = Portal::Accounting::Objects::Ledger->new(dbHandle => $self->dbHandle, langObj => $self->langObj);
  $lo->set('code',$lc);
  $lo->select;
  if ($lo->error) { $self->error($lo->errorMessage); return $self; }
  $self->{ledger} = $lo;
  my $l = $self->langObj;
  if ($args{debug} == 1) { $self->debug("<p>debug on</p>\n"); }

  $self->{data}->{main} = {
    "app" =>  {-Type => "hidden", -Value => "Accounting", },
    "state" =>  {-Type => "hidden", -Value => "JournalEntry", },
    "command" =>  {-Type => "hidden", -Value => "edit", },
    "id" => { -Type => "text", -ReadOnly => 1, -Value => '-1', -DataType => "int" },
    "refNum" => { -Type => "hidden", -Value => '-1', }, #used to set id in a href
    "journal" => {-Type => "text", -ReadOnly => 1, -Value => "G", -DataType => "string" },
    "date" => {-Type => "text", -ReadOnly => 1, -DataType => "string" },
    "payee" => {-Type => "text", -ReadOnly => 1, -DataType => "string", -Value => '' },
    "remember" => {-Type => "hidden", -ReadOnly => 1, -Value => "false", -Label => '', -DataType => "string"},
    "trackingId" => {-Type => "hidden", },
    "origId" => {-Type => "hidden", },
    "ledgerCode" => {-Type => "hidden", -Value => $lc, -DataType => "string" },
    "workingYear" => {-Type => "hidden", },
    "lastClose" => {-Type => "hidden", },
    "windowName" =>  {-Type => "hidden", -DataType => "string", },
    "memo" => { -Type => "text", -ReadOnly => 1, -DataType => "string", }
  };
  $self->{css}->{main} = <<"ENDOF";
  .journal { width: 3em; text-align: left; float: left;}
  .remember { width: 2em; text-align: left; float: left;}
  .id { width: 5em; text-align: left; float: left;}
  .date { width: 7em; text-align: left; float: left;}
  .payee { text-align: left; }
  .delete { width: 2em; text-align: center; color: red; background-color: #ffffff; float: right;}
  .headers { font-weight: bold; }
  .memo { clear: both; width: 100%; }
  td { border-width: 1px; border-bottom-style: solid; padding: 0; margin: 0; }
  table { border-collapse: collapse; width: 100%; }
ENDOF

  $self->{headers} = <<"ENDOF";
      <tr style="clear: float;" class="ledgerBGColor">
        <td colspan="6" style="border: 2px solid; ">
          <div class="delete headers printHide" title='#LANG=Delete#'>
            X
          </div>
          <div class="remember headers">
            <a title='#LANG=Remembered#'>#LANG=R#</a>
          </div>
          <div class="journal headers">
            <a title="#LANG=Journal#">#LANG=Jnl#</a>
          </div>
          <div class="id headers">
            #LANG=Id#
          </div>
          <div class="date headers">
            #LANG=Date#
          </div>
          <div class="payee headers">
            #LANG=Payee# / #LANG=Description#
          </div>
        </td>
      </tr>
ENDOF
  $self->{template}->{main} = <<"ENDOF";
  <tr class="ledgerBGColor">
    <td colspan="6" style="border: 2px solid; ">
      <div class="delete printHide" title="#LANG=Delete#" onclick="var id = '#LABEL=refNum#'; if (confirm('#LANG=Delete journal entry#'+' '+id)) { deleteJournalEntry(id, '#LABEL=ledgerCode#'); }">
        X
      </div>
      <div class="remember">
        #FIELD=remember# #LABEL=remember#&nbsp;
      </div>
      <div class="journal">
        #FIELD=journal#
      </div>
      <div class="id">
        <a href="?app=Accounting&state=JournalEntry&command=edit&id=#LABEL=refNum#&ledgerCode=#LABEL=ledgerCode#">#FIELD=id#</a>
      </div>
      <div class="date">
        #FIELD=date#
      </div>
      <div class="payee">
        #FIELD=payee#&nbsp;
      </div>
      <div class="memo">
        #FIELD=memo#
      </div>
    </td>
  </tr>
ENDOF
  $self->{profile}->{main} = {
    required => [ qw(id remember date ledgerCode journal) ],
    optional => [ qw(payee memo) ],
    constraints => {
      id => qr/^-?\d+$/,
      payee => qr/^[^;"*<>|\\]*$/,
      date => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
      journal => qr/^O|G|CP|CSR|CDP|CDR|TX|IS$/,
    },
  };

  $self->{rowObjects} = [];

  #pull all date information needed from ledgerObject
  $self->set('workingYear', $lo->workingYear());
  $self->set('lastClose', $lo->lastClose);

#   $self->{id} = "id";
#   $self->{INSERTorder}[0] = [ qw( ledger_code date journal payee_payer remember ) ] ;  #insert order for db column names
#   $self->{newOnly}[0] = { orig_id => "", ref_num => ''};
#   $self->{newOnly}[1] = { ref_num => ''};
#   $self->{db2data}[0] = { ref_num => 'id', ledger_code => 'ledgerCode', trans_date => 'date', journal => 'journal', payee_payer => 'payee' };  #map database columns to ->data hash, not specified assumes same name
#   $self->{data2db}[0] = reverse %{$self->{db2data}[0]};
#   $self->{db2data}[1] = { parent_num => 'id' };
#   $self->{data2db}[1] = reverse %{$self->{db2data}[0]};
#   $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM journal_entry_tb WHERE ref_num = #FIELD=id#";
#   $self->{INSERT}[0] = "INSERT INTO journal_entry_tb (##COLUMNS##) VALUES (##VALUES##)";
#   $self->{UPDATE}[0] = "UPDATE journal_entry_tb SET ##UPDATE## WHERE ref_num = #FIELD=id#";
#   #remember to use single quotes around ##foo## if needed
   $self->{DELETE}[0] = "DELETE FROM journal_entry_tb WHERE ref_num = #FIELD=id#";
   $self->{DELETE}[1] = "DELETE FROM debitcredit_tb WHERE parent_num = #FIELD=id#";
#   #find out ifis this being referenced from other tables before deleting
#   $self->{preDELETE}[0] = "";
#   #what should we output as the value of the thing just deleted?
#   $self->{postDELETE}[0] = "SELET ref_num||':'||payee_payer FROM journal_entry_tb WHERE ref_num = '#FIELD=id#'";

  $self->{journals} = {
    'G' => $l->map('General'),
    'CSR' => $l->map('Cash Receipts'),
    'CDR' => $l->map('Credit Receipts'),
    'CP' => $l->map('Cash Purchases'),
    'CDP' => $l->map('Credit Purchases'),
    'TX' => $l->map('Transfers'),
    'IS' => $l->map('Investments'),
    };

  return $self;
}

=item scalar populate(id, ledgerCode, journal, date, payee, remember, rowObjects, trackingId, origId)

  Fills in the data quickly for the object
    trackingId = the genSysId value from an XML JournalEntry Import. Can be "".
    origId = the original id value for this entry.  Can be "".

=cut
sub populate
{
  my $self = shift;
  my %args = ( id => -1, rows => 0, @_ );
  if (defined $args{rowObjects})
  {
    foreach my $row (@{$self->{rowObjects}})
    {
      $self->addRow($row);
    }
  }
  else { $self->{rowObjects} = []; }
  if (defined $args{rows} && $args{rows} =~ /^\d+$/)
  {
    for (my $i = 0; $i < $args{rows}; $i++)
    {
      my $subA = -1;
      #create row obj and populate it
      my $obj = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get("ledgerCode"), debug => $self->{debug});
      if ($obj->error) { $self->error($obj->errorMessage); }
      if ($args{"subAccount_$i"} ne "") { $subA = $args{"subAccount_$i"}; }
      $obj->populate(
        counter => $i,
        id => $args{'id'},
        date => $args{'date'},
        comment => $args{"comment_$i"},
        account => $args{"account_$i"},
        subAccount => $subA,
        subEntity => $args{"subEntity_$i"},
        dorc => $args{"dorc_$i"},
        amount => $args{"amount_$i"},
        cleared => $args{"cleared_$i"},
      );
      $self->debug("<div>rows $i: ".$args{'dorc_'.$i}."</div>");
      if ($obj->error) { $self->error($obj->errorMessage); }
      if ($obj->debug) { $self->debug($obj->debugString); }
      push @{$self->{rowObjects}}, $obj;
    }
  }
  else
  {
    $self->error("<div>Invalid rows: ".$self->get("rows")."</div>");
  }

  return $self->SUPER::populate(@_);
}

# isValid
# validates the object in its current state
sub isValid
{
  my $self = shift;
  # Can we call the parent at the bottom to pick up on the error here without repeating code
  if (!$self->SUPER::isValid())
  {
    $self->error('');
    return 0;
  }

  my $p = $self->get('payee');
  if ($p =~ /[;]/)  #THIS DOES NOT WORK.  DRIVING ME NUTS
  {
    $self->invalid('payee',$p,"<div>; is not allowed</div>");
  }
  if (scalar @{ $self->{rowObjects} } < 2)
  {
    $self->error("<div>The JournalEntry rows array contains too few items (" . (scalar @{ $self->{rowObjects} }) . ")!</div>");
  }
  my $sum = 0;
  my $debit_sum = 0;
  my $credit_sum = 0;
  for (my $rowcount=0; $rowcount < scalar @{ $self->{rowObjects} }; $rowcount++)
  {
    if (ref($self->{rowObjects}[$rowcount]) ne "Portal::Accounting::Objects::JournalEntry::Row")
    {
      $self->invalid('row',$rowcount,"<div>Row $rowcount is not a valid JournalEntry::Row (" . ref($self->{rowObjects}[$rowcount]) . ")!</div>");
    }
    elsif (!$self->{rowObjects}[$rowcount]->isValid())
    {
      $self->invalid('row', $rowcount, "<div>Row $rowcount has invalid data!</div>" . $self->{rowObjects}[$rowcount]->errorMessage);
    }
    else
    {
      my $dorc = $self->{rowObjects}[$rowcount]->get("dorc");
      if ($dorc eq "true" || $dorc eq "1" || $dorc eq 't')
      {
        $debit_sum += $self->{rowObjects}[$rowcount]->get("amount");
      }
      elsif ($dorc eq "false" || $dorc eq "0" || $dorc eq 'f')
      {
        $credit_sum += $self->{rowObjects}[$rowcount]->get("amount");
      }
      else
      {
        $self->invalid('dorc',$rowcount,"<div>Invalid dorc field: $dorc</div>");
      }
    }
  }
  $sum = sprintf("%.2f", $debit_sum - $credit_sum);
  if ($sum != 0.00)
  {
    if ($debit_sum == 0.00)
    {
      $self->invalid('Debits','0', "<div>No debits entered!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum."</div>");
    }
    elsif ($credit_sum == 0.00)
    {
      $self->invalid("Credits", 0, "<div>No credits entered!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum."</div>");
    }
    if ($sum < 0.00)
    {
      $self->invalid('Credits', '', "<div>Credits are greater than debits!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum."</div>");
    }
    elsif ($sum > 0.00)
    {
      $self->invalid('Debits','', "<div>Debits are greater than credits!<br>Debit Sum: " . $debit_sum . "<br>Credit Sum: " . $credit_sum . "<br>Sum: " . $sum."</div>");
    }
  }
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item scalar addRow(rowObject)
  OR (account =>, dorc =>, amount =>, [comment =>, subEntity =>, subAccount =>, cleared =>])

    Adds a row to the rowObjects array in the JournalEntry.
    Returns the row number of the newly added row or undef on failure.
    takes a rowObject with the proper data or a list of data to create a row.

=cut
sub addRow
{
  my $self = shift;
  my $transrowobj;
  if (@_ == 1)
  {
    #a row object was sent. check it and use it.
    $transrowobj = @_[0];
    if (! ref($transrowobj) eq "Portal::Accounting::Objects::JournalEntry::Row") { $self->error("<div>row is not a JournalEntry::Row</div>"); return undef; }
    $transrowobj->set('date', $self->get("date"));
    $transrowobj->set('counter', scalar @{$self->{rowObjects}});
  }
  else
  {
    # create a new JournalEntry::Row with the values passed in
    $transrowobj = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->{langObj}, ledgerCode => $self->get("ledgerCode"), dbHandle => $self->{dbHandle});
    $transrowobj->populate(@_, date => $self->get("date"), ledgerCode => $self->get('ledgerCode'), counter => scalar @{$self->{rowObjects}} );
  }
  if ($transrowobj->error())
  {
    $self->error("<div>Failed to add row!</div>" . $transrowobj->errorMessage);
    return undef;
  }
  if ($transrowobj->isValid)
  {
    push (@{$self->{rowObjects}}, $transrowobj);
    return scalar @{$self->{rowObjects}} - 1;
  }
  $self->error($transrowobj->genErrorString("all"));
  return undef;
}

=item scalar deleteRow(row)

    Removes the row specified from the rowObjects array.

=cut
sub deleteRow
{
  my $self = shift;
  my %args = ( @_ );
  my $row = $args{'row'};

  if (not exists $args{'row'})
  {
    $self->error( "<div>Row number value argument missing in deleteRow!</div>");
    return 0;
  }
  if ($row !~ /^[0-9]+$/)
  {
    $self->error( "<div>Invalid row number argument in deleteRow!  The value does not appear to be numeric.</div>");
    return 0;
  }
  if ($row > 0 && $row < scalar @{$self->{rowObjects}})
  {
    # remove the row from the array using splice to "cut" the element out
    splice(@{$self->{rowObjects}}, $row, 1);
  }
  else
  {
    $self->error( "<div>Invalid row number!  That row does not exist.</div>");
    return 0;
  }
  return 1;
}

=item boolean setDate('2004-9-9')

  Returns 0 for failure and sets error
  1 for success

  Will set all dates in rows and in object

=cut
sub setDate
{
  my $self = shift;
  my $date = shift;
  if ($date =~ /^(\d{4})[\-\\\/](\d{1,2})[\-\\\/](\d{1,2})$/)
  {
    $date = $1 ."-". sprintf("%02d", $2) ."-". sprintf("%02d", $3);
  }
  else
  {
    $self->error("<div>Invalid date: $date</div>");
    return 0;
  }
  $self->{data}->{main}->{date}->{-Value} = $date;
  foreach my $row (@{$self->{rowObjects}})
  {
    $row->set('date',$date);
  }
  return 1;
}

# checkDate
#returns :  true/false
# checks the current date in transaction against last closeout.  if date is before or equal to closeout. it returns false.
sub checkDate
{
  my $self = shift;
  #check dates to make sure user is not entering into closed out books
  my $closeout = $self->{ledger}->lastClose;
  my $date = $self->get('date');
  if ($closeout ne '') #if not closed out, any date can be entered, but if closeout then check date
  {
    #fix formats?
    my @date = split('-', $date);
    my @closeout = split('-', $closeout);
    if (Date_to_Days(@date) <= Date_to_Days(@closeout))
    {
      $self->error('<div class="error">Books were closed out for date '.$closeout.'.  Cannot enter data on '.$date.'</div>');
      return 0;
    }
  }
  return 1;
}

=item scalar insert(commit =>)

    Attempts to insert the contents of the JournalEntryObject
    into the database.
    Returns: the reference number for the new transaction or returns undef and sets error on failure.
    requires:
    optional: commit

=cut
sub insert
{
  my $self = shift;
  my %args = ( commit => 0, @_ );
  my $beginSql = '';

  # check the validity of the transaction object
  if ($self->isValid())
  {
    if ($self->get('journal') eq 'O')
    {
      $self->setDate($self->{ledger}->preOpenBooksDate ); #make sure date is not hacked
      my $id = $self->get('id');
      my $did = $self->openingJournalEntry;
      if ($did ne '') { $self->error('<div class="error">Invalid!  Entering new Opening balance transaction when one already exists with id: "'.$did.'".</div>'); return undef; }
    }
    if (!$self->checkDate) { return undef; }
    if ($self->get('id') == -1)
    {
      # get the nextval(journal_entry_tb_ref_num_seq) of the serial sequence for the table
      my $sth = $self->{dbHandle}->read(sql => "SELECT nextval('journal_entry_tb_ref_num_seq')");
      if ($self->{dbHandle}->error())
      {
        $self->error("<div>Failed to get the nextval() of the journal entry sequence!</div>" . $self->{dbHandle}->errorMessage);
        return undef;
      }
      my @info = $sth->fetchrow_array;
      $self->set('id', $info[0]);
    }
    else
    {
      $self->error("<div>Error!  Tried to insert a transaction with id greater than -1: ".$self->get('id').". Perhaps you need to update it instead.</div>");
      return 0;
    }
    # insert the JournalEntryObject data into the database
    $self->{dbHandle}->write(
      sql => "INSERT INTO journal_entry_tb (modified, ref_num, journal, trans_date, payee_payer, remember, ledger_code, memo) values (NOW(), ?, ?, ?, ?, ?, ?, ?)", plug => [
        $self->{data}->{main}->{id}->{-Value},
        $self->{data}->{main}->{journal}->{-Value},
        $self->{data}->{main}->{date}->{-Value},
        $self->{data}->{main}->{payee}->{-Value},
        $self->{data}->{main}->{remember}->{-Value},
        $self->{data}->{main}->{ledgerCode}->{-Value},
        $self->get('memo')
      ]);
    if ($self->{dbHandle}->error())
    {
      $self->error("<div>Failed to insert journal entry!</div>" . $self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return undef;
    }
    foreach my $obj (@{ $self->{rowObjects} })
    {
      $obj->set('id', $self->get('id'));
      $obj->insert(commit => 0);
      if ($obj->error) { $self->error($obj->errorMessage); }
    }
    if ($self->get('journal') eq 'O')
    {
      my $sql = "INSERT INTO beginning_balance_entry_tb (ref_num, ledger_code) VALUES (?, ?)";
      my $id = $self->get('id');
      my $code = $self->get('ledgerCode');
      $self->dbHandle->write(sql => $sql, plug => [$id, $code]);
      if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); }
    }
    if ($args{commit})
    {
      if (!$self->error) { $self->{dbHandle}->commit; }
      else { $self->{dbHandle}->rollback; }
    }
  }
  else
  {
    return undef;
  }

  # return the created reference number from the database
  return $self->get('id');
}

=item scalar update(commit =>)

    Attempts to update the contents of the JournalEntry
    to the database.
    requires: JournalEntryObject to update
    optional:  commit
    returns:  0 on failure, positive (1) value on successfully updating the reference number contained in the JournalEntryObject to the database

=cut
sub update
{
  my $self = shift;
  my %args = ( commit => 1, @_ );
  my $refNum = $self->get('id');
  my $sth;
  if ($refNum == -1)
  {
    $self->error("<div>Invalid object.  Cannot update transaction with id of '-1'!</div>");
    return 0;
  }
  # check the validity of the transaction object
  if ($self->isValid() )
  {
    if (!$self->checkDate) { return undef; }
    if ($self->get('journal') eq 'O')
    {
      $self->setDate($self->{ledger}->preOpenBooksDate ); #make sure date is not hacked
      my $did = $self->openingJournalEntry;
      if ($did ne $refNum) { $self->error('<div class="error">Invalid!  Updating Opening balance transaction "'.$refNum.'" with different id "'.$did.'".</div>'); return undef; }
      #case id = -1
      # subcase did != '' , error
      # subcase did = ''
      #case id > -1
      # subcase did != id, error
      # subcase did == id
    }

    # update the JournalEntryObject data in the database
    $sth = $self->{dbHandle}->write(
      sql => "UPDATE journal_entry_tb SET modified = NOW(), journal = ?, trans_date = ?, payee_payer = ?, remember = ?, memo = ? WHERE ref_num = ?", plug => [
        $self->get("journal"),
        $self->get("date"),
        $self->get("payee"),
        $self->get("remember"),
        $self->get("memo"),
        $self->get("id"),
        ]
      );
    if ($self->{dbHandle}->error())
    {
      $self->error("<div>Failed to update journal entry!</div>" . $self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return 0;
    }
    if ($sth == 0 )
    {
      # this error occurs in the case that the reference number does not exist in the database; thus, there is nothing to
      # update and the completion of the update process is not necessary or valid
      $self->error("<div>Transaction does not exist for id: ".$refNum."<br>$sth</div>");
      $self->{dbHandle}->rollback;
      return 0;
    }
    # remove all rows from the database for the reference number
    $self->{dbHandle}->write(sql => "DELETE FROM debitcredit_tb WHERE parent_num = ?", plug => [ $refNum ]);
    if ($self->{dbHandle}->error())
    {
      $self->error("<div>Failed to delete row entries!</div>" . $self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return 0;
    }
    # insert the rows in the JournalEntryObject->JournalEntry::RowObject(s) into the database
    foreach my $obj (@{ $self->{rowObjects} })
    {
      $obj->insert(commit => 0);
      if ($obj->error) { $self->error($obj->errorMessage); }
    }

    if ($args{'commit'} eq "1")
    {
      # finish the transaction
      $self->{dbHandle}->commit;
    }
  }
  else
  {
    return 0;
  }
  return 1;
}

=item boolean select( )

    Retrieves the data from a transaction in the database
    for the internal id and places into the object.
    returns true for valid object, false for invalid

=cut
sub select
{
  my $self = shift;
  my $refNum = $self->get('id');
  if ($refNum < 0)
  {
    $self->error("<div>Invalid reference number!</div>");
    return 0;
  }

  # retrieve the data for the reference number from the database and place it in the JournalEntryObject
  my @info;
  my $sql = "SELECT journal, trans_date, payee_payer, remember, ledger_code, (SELECT COUNT(parent_num) FROM debitcredit_tb WHERE parent_num = ref_num) AS rows, memo FROM journal_entry_tb WHERE ref_num = ?";
  my $sth = $self->{dbHandle}->read(sql => $sql, plug => [ $refNum ]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage);  return undef; }
  if (! (@info = $sth->fetchrow_array))  { $self->warning($self->{langObj}->map("No record found"));  return 0; }
  $self->set('journal',$info[0]);
  $self->set('date', $info[1]);
  $self->set('payee', $info[2]);
  my $r = $info[3];
  if ($info[3] eq '1' || $info[3] eq 't') { $r = 'true'; }
  elsif ($info[3] eq '0' || $info[3] eq 'f') { $r = 'false'; }
  $self->set('remember', $r);
  $self->set('ledgerCode', $info[4]);
  $self->set('memo', $info[6]);
  my $i;
  for ( $i = 0 ; $i < $info[5]; $i++)
  {
    my $n = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->{langObj}, ledgerCode => $self->get("ledgerCode"), dbHandle => $self->{dbHandle}, debug => $self->debug, imageDir => $self->{imageDir});
    $n->set("id", $refNum);
    $n->set("counter", $i);
    if ($self->debug){ $self->debug("<p>id: $refNum<br>counter: $i</p>\n");}
    $n->select();
    if ($n->error()) { $self->error($n->errorMessage); }
    if ($self->debug){ $self->debug($n->debugString);}
    $self->{rowObjects}[$i] = $n;
  }
#   if ($info[7] < 2) { $self->error("<p>Invalid object: $refNum!  Must have at least 2 rows!</p>"); }
  return $self->isValid;
}

=item string json()

  This routine will convert this object to a javascript JSON object.

=cut
sub json
{
  use JSON;
  my $self = shift;
  my $var = shift;
  if (! defined $var) { $var = "JournalEntry"; }
  my $j = { id => $self->get('id'),  STATUS => $self->get('id'),
    date => $self->get('date'),
    payee => $self->get('payee'),
    remember => $self->get('remember'),
    journal => $self->get('journal'),
    ledgerCode => $self->get('ledgerCode'),
    memo => $self->get('memo'),
    origId => $self->get('origId'),
    trackingId => $self->get('trackingId'),
  };

  for (my $rowcount=0; $rowcount < scalar @{ $self->rowObjects }; $rowcount++)
  {
    $j->{rows}->[$rowcount] = $self->{rowObjects}->[$rowcount]->hash();
  }
  return "obj = (".objToJson($j).");";
}

sub selectPayeeOptions
{
  my $self = shift;
  if ($self->get('ledgerCode') eq '') { $self->error("<div>Invalid ledger code</div>"); return undef; }
  $self->{LIST}[0] = "SELECT DISTINCT(payee_payer), payee_payer FROM journal_entry_tb WHERE ledger_code = '#FIELD=ledgerCode#' ORDER BY payee_payer";
  return $self->SUPER::selectOptions(@_);
}

sub printHTML
{
  my $self = shift;
  my $r = "<table border='1' cellpadding='0' cellspacing='0'><tr>\n";
  $r .= "<td>$self->{data}->{main}->{id}->{-Value}</td>";
  $r .= "<td>$self->{data}->{main}->{remember}->{-Value}</td>";
  $r .= "<td>$self->{data}->{main}->{date}->{-Value}</td>";
  $r .= "<td>$self->{data}->{main}->{payee}->{-Value}</td>";
  $r .= "<td>$self->{data}->{main}->{ledgerCode}->{-Value}</td>";
  $r .= "</tr></table>";
  $r .= "<p>Rows:" . scalar @{$self->{rowObjects}} ."</p>\n<table border='1' cellpadding='0' cellspacing='0'>";
  my $i = 0;
  foreach my $row (@{$self->rowObjects})
  {
    $r .= $row->printHTML();
  }
  return $r;
}

sub printXML
{
  my $self = shift;
  my $r = $self->SUPER::printXML(@_);
  $r .= '<rows>'."\n";
  my $i = 0;
  foreach my $row (@{$self->rowObjects})
  {
    $r .= $row->printXML();
  }
  $r .= '</rows>'."\n";
  return $r;
}

sub isNew
{
  my $self = shift;
  return ($self->get('id') eq '-1');
}

sub printForm
{
  my $self = shift;
  my %args = (form => 'edit_', @_);
  if (ref $self ne 'Portal::Accounting::Objects::JournalEntry')
  {
    return $self->SUPER::printForm(@_);
  }
  if (defined $self->{data}->{main}->{refNum}) { $self->{data}->{main}->{refNum}->{-Label} = $self->get('id'); }
  $self->{data}->{main}->{ledgerCode}->{-Label} = $self->get('ledgerCode');
  if ($self->get('journal') ne 'O' && $self->get('remember') eq 'true') { $self->{data}->{main}->{remember}->{-Label} = "+"; }
  my %form = $self->SUPER::printForm(name => 'edit_'.$self->get('id'), @_ );
  if (!$self->isNew && !$self->isValid)
  {
    $form{body} .= $self->errorMessage;
  }
  for (my $rowcount=0; $rowcount < scalar @{ $self->{rowObjects} }; $rowcount++)
  {
    my %new = $self->{rowObjects}[$rowcount]->printForm();
    if ($self->{rowObjects}[$rowcount]->error) { $self->error($self->{rowObjects}[$rowcount]->errorMessage); }
    else { $form{body} .= $new{body}; }
    if ($self->debug) { $self->debug($self->{rowObjects}[$rowcount]->debugString); }

  }
  $form{body} = "\n<table id=\"j".$self->get('id').'">'."\n".$form{body}."\n</table>\n\n";
  return %form;
}

sub printHeaders
{
  my $self = shift;
  my %form;
  $form{body} = '<table><thead>'.$self->{headers};
  my $row;
  if (scalar @{$self->{rowObjects}} > 0)
  {
    $row = $self->{rowObjects}[0];
  }
  else
  {
    $row = Portal::Accounting::Objects::JournalEntry::Row->new(langObj => $self->langObj, dbHandle => $self->dbHandle, ledgerCode => $self->get('ledgerCode'));
  }
  $form{body} .= $row->{headers}."</thead></table>\n";
  $form{css} = $self->{css}->{main} . $row->{css}->{main};
  $form{body} = $self->langSubstitute($form{body});
  return %form;
}

=item delete(system => 0|1)

  deletes the journal entry with the id of the internal id field.
  admin - flag to tell system that users are deleting or the system is internally deleting such as from the investment system

=cut
sub delete
{
  my $self = shift;
  my %args = (system => 0, @_);
  #used to check for valid deletable journals
  my %journals = (
    'CP' => '',
    'CSR' => '',
    'G' => '',
    'TX' => '',
    'CDR' => '',
    'CDP' => '',
  );
  if (!$args{system})
  {
    my $sql = "SELECT journal FROM journal_entry_tb WHERE ref_num = ".$self->get('id');
    my $sth = $self->{dbHandle}->read($sql);
    if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
    if (my @info = $sth->fetchrow_array)
    {
      if (! defined $journals{$info[0]}) { $self->error('<div class="error">Cannot delete from journal '.$info[0]."</div>"); return -1; }
    }
    else { $self->error('<div class="error">no record found while checking for journal</div>'); return undef; }
  }
  return $self->SUPER::delete(@_, commit => !$args{system});
}

=item integer openingJournalEntry()

  This will look for a transaction that specifies the opening balances
  for a given ledger.  If no Journal Entry is defined nothing is returned.
  Otherwise the number of the Journal Entry is returned.

=cut
sub openingJournalEntry
{
  my $self = shift;
  my $je = '';
  my $l = $self->get('ledgerCode');
  if ($l eq '') { $self->error('<div class="error">No ledgerCode sent to openingJournalEntry!</div>'); return undef; }
  my $sql = "SELECT ref_num FROM beginning_balance_entry_tb WHERE ledger_code = ?";
  my $sth = $self->dbHandle->read(sql => $sql, plug => [$l] );
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  if (my @data = $sth->fetchrow_array) { $je = @data[0]; }
  return $je;
}

1;
__END__

=pod

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

James A. Pattie (mailto:james@pcxperience.com), Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1), JournalEntry(3), DBIWrapper(3)

=cut
