# JournalEntry::Row.pm - Contains the data for a debitcredit_tb table row.
# Created by Jason A. Pattie, 11/07/2000.
#$Id: Row.pm,v 1.5 2006/05/12 14:35:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::JournalEntry::Row;
use Portal::Accounting::Objects::BaseL;
use strict;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;
@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.5 $';

=head1 NAME

JournalEntry::Row - Object that contains a row of data used to build a JournalEntry in the accounting database.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::JournalEntry::Row;
  my $transrowobj = JournalEntry::Row->new(ledgerCode => '', langObj => , dbHandle => , );
  $transrowobj->populate(date => '12-14-1999',
                             comment => '',
                             subEntity => '',
                             account => 1001,
                             subAccount => -1,
                             dorc => 'true',
                             amount => '2579.63',
                             cleared => 'r');
  if ($transrowobj->error())
  {
    die $transrowobj->errorMessage();
  }

  # this object is generally used to build an array passed to
  # the constructor of a JournalEntryObject
  my @rowObjectArray;
  $rowObjectArray[0] = $transrowobj;
  $rowObjectArray[1] = $transrowobj;
  $rowObjectArray[1]->set('dorc','false');

  # check for validness of row objects values
  # before adding to JournalEntryObject
  if ($rowObjectArray[1]->isValid())
  {
    ...
  }
  else
  {
    die $rowObjectArray[1]->errorMessage();
  }

=head1 DESCRIPTION

JournalEntry::Row is used as a container class that is incorporated
into the Objects::JournalEntry class.

=head1 Exported FUNCTIONS

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = (ledgerCode => '', debug => 0, dbHandle => undef, @_);
  if ($args{debug} == 1) { $self->debug("<p>row debug on</p>\n"); }
  $self->{dbHandle} = $args{dbHandle};
  $self->{id} = "id";
  $self->{table} = "debitcredit_tb";
  $self->{INSERTorder}[0] = [ qw( parent_num counter ledger_code trans_date account_num sub_account_num sub_e_code comment dorc amount c_amount d_amount cleared ) ] ;  #insert order for db column names
  $self->{newOnly}[0] = { parent_num => '', counter => ''};
  $self->{db2data}[0] = { 'parent_num' => 'id', 'ledger_code' => 'ledgerCode', 'trans_date' => 'date',
    'account_num' => 'account', 'sub_account_num' => 'subAccount', 'sub_e_code' => 'subEntity', 'comment' => 'comment',
  };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};
  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM debitcredit_tb WHERE parent_num = #FIELD=id# AND counter = #FIELD=counter#";
  $self->{INSERT}[0] = "INSERT INTO debitcredit_tb (##COLUMNS##) VALUES (##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE debitcredit_tb SET ##UPDATE## WHERE parent_num = #FIELD=id# AND counter = #FIELD=counter#";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[0] = "DELETE FROM debitcredit_tb WHERE parent_num = #FIELD=id# AND counter = #FIELD=counter#";

  #account and sub account are hidden and the label is used to display names instead of numbers
  $self->{data}->{main} = {
    "id" => { -Label => "Id", -Type => "hidden", -Value => -1, -DataType => "int" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}},
    "amount" => { -Type => "calculator", -DataType => "float",
      -WidgetOptions => { calcButton => 0, undoButton => 0, helpLink => 0 }
    },
    "subAccount" => { -Type => "hidden", -DataType => "int", -Label => ' ', -Value => -1, },
    "account"    => { -Type => "hidden", -DataType => "int", -Label => " ", },
    "date"       => { -Type => "text", -ReadOnly => 1, -DataType => "string", },
    "comment"    => { -Type => "text", -ReadOnly => 1, -DataType => "string", -Value => "", },
    "dorc"       => { -Type => "text", -ReadOnly => 1, -DataType => "string", -Value => 'true',},
    "subEntity"  => { -Type => "text", -ReadOnly => 1, -DataType => "string", -Value => '', },
    "cleared"    => { -Type => "text", -ReadOnly => 1, -DataType => "string", -Value => "_" },
    "counter"    => { -Type => "text", -ReadOnly => 1, -DataType => "int", },
    #debit and credit labels are set by printForm
    "debit"      => { -Type => "hidden", -ReadOnly => 1, -DataType => "float", -Value => "", -Label => '', },
    "credit"     => { -Type => "hidden", -ReadOnly => 1, -DataType => "float", -Value => "", -Label => '', },
    "c_amount"   => { -Type => "hidden", },
    "d_amount"   => { -Type => "hidden", },
  };
  $self->{css}->{main} = <<"ENDOF";
  .subEntity { width: 3em; text-align: left; }
  .account { width: 15em; text-align: left; }
  .subAccount { width: 11em; text-align: left; }
  .comment { text-align: left; }
  .debit { width: 6em; text-align: right;  }
  .credit { width: 6em; text-align: right;  }
  .check { color: #FF0011; }
  .deposit { color: #00BB33; }
ENDOF
  $self->{headers} = <<"ENDOF";
  <tr>
    <th class="subEntity headers">
      <a title="#LANG=Sub Entity#">S</a>
    </th>
    <th class="account headers">
      #LANG=Account#
    </th>
    <th class="subAccount headers">
      #LANG=Sub Account#
    </th>
    <th class="debit headers">
      #LANG=Debit#
    </th>
    <th class="credit headers">
      #LANG=Credit#
    </th>
    <th class="comment headers">
      &nbsp;#LANG=Comment#
    </th>
  </tr>
ENDOF
  $self->{template}->{main} = <<"ENDOF";
  <tr>
    <td class="subEntity">
      #FIELD=subEntity#&nbsp;
    </td>
    <td class="account">
      #LABEL=account# #FIELD=account#
    </td>
    <td class="subAccount">
      #LABEL=subAccount# #FIELD=subAccount#&nbsp;
    </td>
    <td class="debit">
      &nbsp;#FIELD=debit# #LABEL=debit#
    </td>
    <td class="credit">
      &nbsp;#FIELD=credit# #LABEL=credit#
    </td>
    <td class="comment">
      &nbsp;#FIELD=comment# #FIELD=d_amount# #FIELD=c_amount#
    </td>
  </tr>
ENDOF
  $self->{profile}->{main} = {
    required => [ qw(account dorc amount ) ],
    optional => [ qw(subAccount comment subEntity cleared) ],
    constraints => {
      comment => qr/[^"].*/,
      amount => qr/^([0-9]*)(\.[0-9]{1,2})?$/,
      account => qr/^\d+$/,
      subAccount => qr/^-?\d+$/,
      subEntity => qr/^[\d\w]+$/,
      cleared => qr/^[_xr]$/,
    },
  };
  return $self;
}

=item   scalar isValid(void)

    Runs through the data fields contained in the class to make sure
    they contain valid data.  This function will be called
    automatically when embedded in a JournalEntryObject.

=cut
sub isValid
{
  my $self = shift;
  if (!$self->SUPER::isValid())
  {
    $self->error("<div>skipping rest of isValid</div>");
    return undef;
  }

  if (!defined $self->{dbHandle})
  {
    $self->missing("dbHandle");
  }
  my $dorc = $self->get('dorc');
  if ($dorc eq 't' || $dorc eq '1') { $self->set('dorc','true'); }
  elsif ($dorc eq 'f' || $dorc eq '0') { $self->set('dorc','false'); }
  elsif ($dorc ne "true" && $dorc ne "false")
  {
    $self->invalid("dorc", $dorc);
  }
  if ($self->get("d_amount") !~ /^-?([0-9]+)(\.[0-9]{2})?$/)
  {
    $self->invalid("d_amount", $self->get("d_amount"));
  }
  if ($self->get("c_amount") !~ /^-?([0-9]+)(\.[0-9]{2})?$/)
  {
    $self->invalid("c_amount", $self->get("c_amount"));
  }

  if ($self->{error})
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }
  return 1;
}

=item boolean populate()

  requires:  date, account, dorc, amount
  optional:  comment, subEntity, subAccount, cleared

    Fills in this instance of the Portal::Accounting::Objects::JournalEntry::Row
    and validates the values contained in it.

=cut
sub populate
{
  my $self = shift;
  $self->SUPER::populate(@_);
  if ($self->error) { return 0; }

  my $account = $self->get("account");
  my $amount = $self->get("amount");
  if ($account =~ /^1/)
  {
    #!!! what if this is a cash account?  the c_amounts are different?
    if ($self->get("dorc") eq 'true')
    {
      $self->{data}->{main}->{c_amount}->{-Value}  -= $amount;  #increasing asset decreases cash
      $self->{data}->{main}->{d_amount}->{-Value}  = $amount;  #debits increase assets
    }
    else
    {
      $self->{data}->{main}->{c_amount}->{-Value}  = $amount; #decreasing asset like selling which increases cash
      $self->{data}->{main}->{d_amount}->{-Value}  -= $amount; #credits decrease assets
    }
  }
  elsif ($account =~ /^2/)
  {
    if ($self->get("dorc") eq 'true')
    {
      $self->{data}->{main}->{d_amount}->{-Value}  -= $amount; #debit decrease liability
      $self->{data}->{main}->{c_amount}->{-Value}  -= $amount; #decreasing liability uses cash
    }
    else
    {
      $self->{data}->{main}->{d_amount}->{-Value}  = $amount; #decreasing liability uses cash
      $self->{data}->{main}->{c_amount}->{-Value}  = $amount;  #increasing liability gets cash
    }
  }
  elsif ($account =~ /^3/)
  {
    if ($self->get("dorc") eq 'true')
    {
      $self->{data}->{main}->{d_amount}->{-Value}  -= $amount; #decreasing equity with debit
      $self->{data}->{main}->{c_amount}->{-Value}  -= $amount; #decreasing equity uses cash
    }
    else
    {
      $self->{data}->{main}->{d_amount}->{-Value}  = $amount; #increasing equity with credit
      $self->{data}->{main}->{c_amount}->{-Value}  = $amount; #increasing equity gets cash
    }
  }
  elsif ($account =~ /^4/)  #same as 3
  {
    if ($self->get("dorc") eq 'true')
    {
      $self->{data}->{main}->{d_amount}->{-Value}  -= $amount; #decreasing revenue with debit
      $self->{data}->{main}->{c_amount}->{-Value}  -= $amount; #decreasing revenues uses cash
    }
    else
    {
      $self->{data}->{main}->{d_amount}->{-Value}  = $amount; #increasing revenue with credit
      $self->{data}->{main}->{c_amount}->{-Value}  = $amount; #increasing revenues increases cash
    }
  }
  elsif ($account =~ /^5/)  #same as 1
  {
    if ($self->get("dorc") eq 'true')
    {
      $self->{data}->{main}->{d_amount}->{-Value}  = $amount;  #expenses increased by debit
      $self->{data}->{main}->{c_amount}->{-Value}  -= $amount;  #expenses use cash
    }
    else
    {
      $self->{data}->{main}->{d_amount}->{-Value}  -= $amount;  #expenses decreased by credit
      $self->{data}->{main}->{c_amount}->{-Value}  = $amount; #negative expenses refund cash
    }
  }
  else { $self->invalid('account', $account, "<div>account number is invalid.  Must start with 1, 2, 3, 4, or 5</div>") ; }

  if ($self->dataError) { return 0; }
  return 1;
}

sub setDate
{
  my $self = shift;
  my $date = shift;
  if ($date =~ /^(\d{4})[\-\\\/](\d{1,2})[\-\\\/](\d{1,2})$/)
  {
    $date = $1 ."-". sprintf("%02d", $2) ."-". sprintf("%02d", $3);
  }
  else
  {
    $self->invalid("<div>Invalid date: $date</div>");
    return 0;
  }
  $self->{data}->{main}->{date}->{-Value} = $date;
  return 1;
}

=item string javascript(var)

  var is the name of the js variable to use. defaults to JournalEntry

=cut
sub javascript
{
  my $s = shift;
  my $var = shift;
  if (not defined $var) { $var = "JournalEntry"; }
  my $row = $s->get("counter");
  my $r = "  $var.rows[$row] = new Object();\n";
  $r .= "  $var.rows[$row].dorc = '".$s->get("dorc")."';\n";
  $r .= "  $var.rows[$row].account = '".$s->get("account")."';\n";
  $r .= "  $var.rows[$row].subAccount = '".$s->get("subAccount")."';\n";
  $r .= "  $var.rows[$row].subEntity = '".$s->get("subEntity")."';\n";
  $r .= "  $var.rows[$row].comment = '".$s->get("comment")."';\n";
  $r .= "  $var.rows[$row].amount = '".$s->get("amount")."';\n";
  $r .= "  $var.rows[$row].c_amount = '".$s->get("c_amount")."';\n";
  $r .= "  $var.rows[$row].d_amount = '".$s->get("d_amount")."';\n";
  $r .= "  $var.rows[$row].cleared = '".$s->get("cleared")."';\n";
  return $r;
}

sub hash
{
  my $s = shift;
  return {
    dorc => $s->get("dorc"),
    account => $s->get("account"),
    subAccount => $s->get("subAccount"),
    subEntity => $s->get("subEntity"),
    comment => $s->get("comment"),
    amount => $s->get("amount"),
    c_amount => $s->get("c_amount"),
    d_amount => $s->get("d_amount"),
    cleared => $s->get("cleared"),
  };
}

sub printForm
{
  my $self = shift;
  my $dorc = $self->get('dorc');
  my $amount = $self->get('amount');
  if ($dorc eq "true")
  {
    $self->{data}->{main}->{debit}->{-Value} = $amount;
    $self->{data}->{main}->{debit}->{-Label} = $self->commify($amount);
  }
  else
  {
    $self->{data}->{main}->{credit}->{-Value} = $amount;
    $self->{data}->{main}->{credit}->{-Label} = $self->commify($amount);
  }
  return $self->SUPER::printForm(@_);
}

sub isNew
{
  my $self = shift;
  if ($self->get('id') == -1) { return 1; }
  return 0;
}

sub select
{
  my $self = shift;
  my $r = $self->SUPER::select(@_);
  #convert dorc field
  my $dorc = $self->get('dorc');
  if ($dorc eq '0' || $dorc eq 'f')
  {
    $self->set('dorc', 'false');
  }
  elsif ($dorc eq '1' || $dorc eq 't')
  {
    $self->set('dorc', 'true');
  }

  #set account sub account labels to names/codes
  if (!$self->selectNames) { $self->error("<div>could not set account and sub account names</div>"); }
  return $r;
}

#used to set label fields for account and sub account to names
sub selectNames
{
  my $self = shift;
  my $type;
  my $asql = "SELECT name, taxable, type FROM account_tb WHERE number = '".$self->get('account')."'";
  my $sub = $self->get('subAccount');
  my $ssql = "SELECT sub_acc_name AS name FROM sub_acc_tb WHERE sub_acc_number = '$sub'";
  my $result = $self->{dbHandle}->read($asql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  if (my $info = $result->fetchrow_hashref)
  {
    my $flag = '';
    if ($info->{taxable}) { $flag = "*"; }
    $self->{data}->{main}->{account}->{-Label} = $info->{name}."$flag";
    $type = $info->{type};
  }
  if ($type == 0)
  {
    my $class = "check";
    if ($self->get('dorc') eq 'true') { $class = "deposit"; }
    if ($sub != -1) { $self->{data}->{main}->{subAccount}->{-Label} = "<span class=\"$class\">$sub</span>"; }
  }
  elsif ($type == 10)
  {
    $self->{data}->{main}->{subAccount}->{-Label} = "<span class=\"invoice\">$sub</span>";
  }
  else
  {
    $result = $self->{dbHandle}->read($ssql);
    if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); }
    elsif (my $info = $result->fetchrow_hashref)
    {
      $self->{data}->{main}->{subAccount}->{-Label} = $info->{name};
    }
  }
  if ($self->error) { return 0; }
  return 1;
}

1;
__END__

=pod

   NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

JT Moree (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), JournalEntryObject(3)

=cut
