# OpenBooks.pm - The Object Class that provides a OpenBooks Object
#$Id: OpenBooks.pm,v 1.6 2006/05/04 18:46:55 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::OpenBooks;
use strict;
use Portal::Accounting::Objects::BaseL;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.6 $';

=head1 NAME

OpenBooks - Object used to build a OpenBooks Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::OpenBooks;
  my $OpenBooksObj = Portal::Accounting::Objects::OpenBooks->new(   );
  if ($OpenBooksObj->error())
  {
    die $OpenBooksObj->errorMessage();
  }

=head1 DESCRIPTION

OpenBooks is a OpenBooks class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::OpenBooks.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );
  if ($self->error) { return $self; }
  $self->{dbHandle} = $args{dbHandle};

  # instantiate anything unique to this module
  my $phrases = $self->{langObj};
  $self->{data}->{main} = {
    "date" => {
      -Label => $phrases->map("cumulativeDates"),
      -Type => "select",
      -DataType => "string",
      -Options => $self->{formObj}->createSelectOptions($self->getCloseOutDates)
    },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string", },
    "command" => { -Type => "hidden", -Value => "reOpen" },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Ledger" },
    "submit" => { -Type => "submit", -Value => $phrases->map("Submit") },
  };
  if (scalar @{$self->{data}->{main}->{date}->{-Options}->{names}} == 0)
  {
    $self->{data}->{main}->{date} = { -Type => "hidden", -Label => $phrases->map("noDates") },
    $self->{data}->{main}->{submit} = { -Type => "hidden", },
  }
  $self->{template}->{main} = <<"ENDOF";
  <div class="primary">
  #FORMERRORSTRING#
  <form #FORMARGS#>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #LABEL=date#<br>
  #FIELD=date#
  #FIELD=submit#
  </form>
  </div>
ENDOF

  $self->{profile}->{main} = {
    required => [ qw( date ) ],
    optional => [  ],
    constraints  =>
        {
            date   => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
        },
  };

  return $self;
}

sub isNew { return 1; }

=item bool populate();

  this method inserts the data given into the object.

=cut

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item string update(commit => 1)

 Reopens a ledger on the date given when populated.
 Returns string on success, undef on errors and sets the errorMessage.

=cut

sub update
{
  my $self = shift;
  my %args = ( commit => 1, @_ );
  my $ledgerCode = $self->get('ledgerCode');
  my $reopenDate = $self->get('date');
  my $lookupYear = (split "-", $reopenDate)[0];
  my $toDate;

  # remove any balance table entries that exist on or after the given reopenDate period
  my $sth = $self->{dbHandle}->write(sql => "DELETE FROM closeouts WHERE date >= ? AND ledger_code = ?", plug => [ $reopenDate, $ledgerCode ]);
  if ($self->{dbHandle}->error)
  {
    $self->error(sprintf($self->{langObj}->map("errorRemovingAccountBalances"), $lookupYear) . "<br>\n" . $self->{dbHandle}->errorMessage);
    return undef;
  }

  my $sth = $self->{dbHandle}->write(sql => "UPDATE known_years_tb SET closed = false WHERE year >= ? AND ledger_code = ?", plug => [ $lookupYear, $ledgerCode ]);
  if ($self->{dbHandle}->error)
  {
    $self->error(sprintf($self->{langObj}->map("errorOpeningFiscalYears"), $lookupYear) . "<br>\n" . $self->{dbHandle}->errorMessage);
    return undef;
  }

  # commit the work
  if ($args{commit} == 1)
  {
    $sth = $self->{dbHandle}->commit;
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{langObj}->map("errorCommittingTransaction") . "<br>\n" . $self->{dbHandle}->errorMessage);
      return undef;
    }
  }
  my $p = $self->{langObj};
  return $p->map("Success") ." ". $p->map("Open Books") . " $reopenDate $ledgerCode ";
}

sub getCloseOutDates
{
  my $self = shift;
  my $ledgerCode = $self->get('ledgerCode');
  #selecting name/value pairs,  in this case it is same data
  my $sql = "SELECT date, date FROM closeouts WHERE ledger_code = '$ledgerCode' ORDER BY date DESC";
  my $sth = $self->{dbHandle}->read($sql);
  return $sth;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
